//+------------------------------------------------------------------+
//|                                                         VQzz.mq4 |
//|                                          Copyright  2011 zznbrm |
//| Updated original VQ.mq4 by raff1410@o2.pl                        |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011 zznbrm"
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 FireBrick
#property indicator_width1 3
#property indicator_color2 Green
#property indicator_width2 3
#property indicator_maximum 1
#property indicator_minimum 0

//+------------------------------------------------------------------+
//| User Inputs                                                      |
//+------------------------------------------------------------------+
extern int eintTF = 0;
extern int eintPeriod = 7;
extern int eintMethod = MODE_LWMA;
extern int eintSmoothing = 1;
extern int eintFilter = 5;

//+------------------------------------------------------------------+
//| Indicator Buffers                                                |
//+------------------------------------------------------------------+
double gadblDown[];
double gadblUp[];
double gadblVQ[];
double gadblPrev[];

//+------------------------------------------------------------------+
//| Program Variables                                                |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| init()                                                           |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers( 4 ); 
   IndicatorDigits( Digits );
   
   string strName = "VQzz-";
   
   if ( eintTF == 0 )     strName = strName +"  TF="+ Period()+"   ";
   else                   strName = strName +"  TF="+ eintTF+"   ";
    
   IndicatorShortName( strName );
   
   SetIndexBuffer( 0, gadblDown );   SetIndexStyle( 0, DRAW_HISTOGRAM );  SetIndexLabel( 0, NULL );
   SetIndexBuffer( 1, gadblUp );     SetIndexStyle( 1, DRAW_HISTOGRAM );  SetIndexLabel( 1, NULL ); 
   SetIndexBuffer( 2, gadblVQ );     SetIndexStyle( 2, DRAW_NONE );       SetIndexLabel( 2, strName ); 
   SetIndexBuffer( 3, gadblPrev );   SetIndexStyle( 3, DRAW_NONE );       SetIndexLabel( 3, NULL );

   return( 0 );
}

//+------------------------------------------------------------------+
//| deinit()                                                         |
//+------------------------------------------------------------------+
int deinit()
{   
   //Comment( "" );
   return( 0 );
}

//+------------------------------------------------------------------+
//| start()                                                          |
//+------------------------------------------------------------------+
int start()
{
   //int intStart = GetTickCount();
   int intReturn;
   
   if ( eintTF == 0 || eintTF == Period() )       intReturn = processCurrTf();
   else                                           intReturn = processOtherTf();
      
   //Comment( GetTickCount() - intStart );
   return( intReturn );
}

//+------------------------------------------------------------------+
//| processCurrTf                                                    |
//+------------------------------------------------------------------+ 
int processCurrTf()
{
   int intCountedBars = IndicatorCounted();

   if ( intCountedBars < 0 )   return( -1 );   
   if ( intCountedBars > 0 )   intCountedBars--;   
   if ( intCountedBars == 0 )  intCountedBars = ( eintPeriod + 2 + eintSmoothing );
   
   int intMax = Bars - intCountedBars;
   double dblCurr;
   
   for ( int inx = intMax; inx >= 0; inx-- )
   {
      dblCurr = calcUtfVQ( inx );
      
      if ( dblCurr == 0.0 )
      {
         if ( gadblVQ[inx+1] < 0.0 )     dblCurr = (-1) * Point;
         else                            dblCurr = Point;
      }
      
      if ( MathAbs( dblCurr ) < ( eintFilter * Point ) )      gadblVQ[inx] = gadblVQ[inx+1];
      else                                                    gadblVQ[inx] = dblCurr;
                                       
      gadblDown[inx] = 0.0;
      gadblUp[inx] = 0.0;
      
      if ( gadblVQ[inx] > 0.0 )         gadblUp[inx] = 1.0;
      else if ( gadblVQ[inx] < 0.0 )    gadblDown[inx] = 1.0;
   }
   
   return( 0 );
}

//+------------------------------------------------------------------+
//| processOtherTf                                                   |
//+------------------------------------------------------------------+ 
int processOtherTf()
{
   int intCountedBars = IndicatorCounted();

   if ( intCountedBars < 0 )   return( -1 );   
   if ( intCountedBars > 0 )   intCountedBars--;   
   if ( intCountedBars == 0 )  intCountedBars = ( eintPeriod + 2 + eintSmoothing );
   
   double dblCurr;
   datetime dtCurr; 
   int intShift;  
   int intMax = MathMax( Bars - intCountedBars, eintTF / Period() );
      
   for ( int inx = intMax; inx >= 0; inx-- )
   {
      intShift = iBarShift( Symbol(), eintTF, Time[inx] ); 
      dtCurr = iTime( Symbol(), eintTF, intShift ); 
      gadblPrev[inx] = getPrev( inx+1, dtCurr );    
      dblCurr = calcUtfVQ( intShift );
      
      if ( dblCurr == 0.0 )
      {
         if ( gadblPrev[inx] < 0.0 )     dblCurr = (-1) * Point;
         else                            dblCurr = Point;
      }
      
      if ( MathAbs( dblCurr ) < ( eintFilter * Point ) )   gadblVQ[inx] = gadblPrev[inx];
      else                                                 gadblVQ[inx] = dblCurr;
                                       
      gadblDown[inx] = 0.0;
      gadblUp[inx] = 0.0;
      
      if ( gadblVQ[inx] > 0.0 )         gadblUp[inx] = 1.0;
      else if ( gadblVQ[inx] < 0.0 )    gadblDown[inx] = 1.0;
   }
   
   return( 0 );
}

//+------------------------------------------------------------------+
//| calcUtfVQ                                                        |
//+------------------------------------------------------------------+ 
double calcUtfVQ( int inx )
{
   double dblHigh = iMA( Symbol(), eintTF, eintPeriod, 0, eintMethod, PRICE_HIGH, inx );
   double dblLow  = iMA( Symbol(), eintTF, eintPeriod, 0, eintMethod, PRICE_LOW, inx );
   double dblOpen = iMA( Symbol(), eintTF, eintPeriod, 0, eintMethod, PRICE_OPEN, inx );
   double dblClose = iMA( Symbol(), eintTF, eintPeriod, 0, eintMethod, PRICE_CLOSE, inx );
   double dblClose2 = iMA( Symbol(), eintTF, eintPeriod, 0, eintMethod, PRICE_CLOSE, inx+eintSmoothing );

   if ( dblHigh == 0.0 || dblLow == 0.0 || dblOpen == 0.0 || dblClose == 0.0 || dblClose2 == 0.0 )
      return( 0.0 );
      
   return( calcVQ( dblHigh, dblLow, dblOpen, dblClose, dblClose2 ) );
}

//+------------------------------------------------------------------+
//| calcVQ                                                           |
//+------------------------------------------------------------------+ 
double calcVQ( double h, double l, double o, double c, double c2 )
{
   double dblMax = MathMax(h-l,MathMax(h-c2,c2-l));
   double dblVQ = MathAbs(((c - c2) / dblMax + (c - o) / (h - l)) * 0.5) * ((c - c2 + (c - o)) * 0.5);
   return( dblVQ );
}

//+------------------------------------------------------------------+
//| getPrev                                                          |
//+------------------------------------------------------------------+ 
double getPrev( int intStart, datetime dt )
{
   double dblPrev = 0.0;
   int inx = intStart;
   
   while( true )
   {
      if ( inx > ( Bars-1 ) )   break;
      
      if ( Time[inx] < dt )
      {
         dblPrev = gadblVQ[inx];
         break;
      }
      
      inx++;
   }

   return( dblPrev );
}